from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Texas raw data for consolidation."""

    postal_code = "TX"
    fields = dict(
        company="JOB_SITE_NAME",
        location="CITY_NAME",
        date="NOTICE_DATE",
        jobs="TOTAL_LAYOFF_NUMBER",
    )
    date_format = "%Y-%m-%d %H:%M:%S"



import typing
from datetime import datetime

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform New Jersey raw data for consolidation."""

    postal_code = "NJ"
    fields = dict(
        company="Company",
        location="City",
        date="Effective Date",
        jobs="Workforce Affected",
    )
    date_format = ["%Y-%m-%d %H:%M:%S", "%M/%d/%Y", "%M/%d/%y"]
    jobs_corrections = {
        "TBA": None,
        "To be Determined": None,
        "-": None,
        23695: None,
        # The United airlines number is legimate, though nationwide
        # https://abcnews.go.com/Politics/united-airlines-furlough-16000-employees/story?id=72771897
        16000: 16000,
    }
    date_corrections = {
        "TBA": None,
        "Temp layoff": None,
        "-": None,
        "3030-08-23 00:00:00": datetime(2020, 8, 23),
    }

    def transform_jobs(self, value: str) -> typing.Optional[int]:
        """Transform a raw jobs number into an integer.

        Args:
            value (str): A raw jobs number provided by the source

        Returns: An integer number ready for consolidation. Or, if the value is invalid, a None.
        """
        # Cut the asterisk they sometimes use
        value = value.replace("*", "")

        # Do the normal stuff
        return super().transform_jobs(value)



import typing
from pathlib import Path

import click

from . import consolidate as consolidate_runner
from . import download as download_runner
from . import utils


@click.group()
def cli():
    """Consolidate, enrich and republish the data gathered by warn-scraper."""
    pass


@cli.command()
@click.option(
    "--download-dir",
    default=utils.WARN_TRANSFORMER_OUTPUT_DIR / "raw",
    type=click.Path(),
    help="The Path were the results will be downloaded",
)
@click.option(
    "--source",
    default=None,
    help="The source to download. Default is all sources.",
)
def download(download_dir: Path, source: typing.Optional[str] = None):
    """Download all the CSVs in the WARN Notice project on biglocalnews.org."""
    download_runner.run(download_dir, source)


@cli.command()
@click.option(
    "--input-dir",
    default=utils.WARN_TRANSFORMER_OUTPUT_DIR / "raw",
    type=click.Path(),
    help="The Path were the raw files results are located",
)
@click.option(
    "--source",
    default=None,
    help="The source to download. Default is all sources.",
)
def consolidate(input_dir: Path, source: typing.Optional[str] = None):
    """Consolidate raw data using a common data schema."""
    consolidate_runner.run(input_dir, source)


if __name__ == "__main__":
    cli()



from datetime import datetime

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Washington D.C. raw data for consolidation."""

    postal_code = "DC"
    fields = dict(
        company="Organization Name",
        location=lambda x: "Washington D.C.",  # Hardcode in the city name
        date="Notice Date",
        jobs="Number toEmployees Affected",
    )
    date_format = "%B %d, %Y"
    date_corrections = {
        "May 2 and 5, 2020": datetime(2020, 5, 2),
        "March, 20, 2020": datetime(2020, 3, 20),
        "31, 2019": datetime(2019, 12, 31),
    }



import typing

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Missouri raw data for consolidation."""

    postal_code = "MO"
    fields = dict(
        company="COMPANY NAME",
        location="LOCATION",
        date="DATE RECEIVED",
        jobs="# AFFECTED",
    )
    date_format = ("%m/%d/%Y", "%m/%d/%y")
    jobs_corrections = {
        "330 remote workers (18 located in Missouri)": 18,
        "Unknown": None,
    }

    def transform_date(self, value: str) -> typing.Optional[str]:
        """Transform a raw date string into a date object.

        Args:
            value (str): The raw date string provided by the source

        Returns: A date object ready for consolidation. Or, if the date string is invalid, a None.
        """
        # A little custom clean up based on the weird stuff from this source
        value = value.strip().split()[0].strip()
        return super().transform_date(value)



from datetime import datetime

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform South Carolina raw data for consolidation."""

    postal_code = "SC"
    fields = dict(
        company="company",
        location="location",
        date="date",
        jobs="jobs",
    )
    date_format = "%m/%d/%Y"
    date_corrections = {
        "4/8/20/20": datetime(2020, 4, 8),
        "12/31//2015": datetime(2015, 12, 31),
    }



from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Virginia raw data for consolidation."""

    postal_code = "VA"
    fields = dict(
        company="Company Name",
        location="Location City",
        date="Notice Date",
        jobs="Employees Affected",
    )
    date_format = "%m/%d/%Y"



import typing

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Ohio raw data for consolidation."""

    postal_code = "OH"
    fields = dict(
        company="Company",
        location="City/County",
        date="DateReceived",
        jobs="Potential NumberAffected",
    )
    date_format = "%m/%d/%Y"

    def transform_date(self, value: str) -> typing.Optional[str]:
        """Transform a raw date string into a date object.

        Args:
            value (str): The raw date string provided by the source

        Returns: A date object ready for consolidation. Or, if the date string is invalid, a None.
        """
        # Cut out cruft
        value = value.replace("Updated", "")
        value = value.replace("Revised", "")
        # Split double dates
        if len(value) == 20:
            value = value[:10]
        # Do the typical stuff
        return super().transform_date(value)



import typing

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Connecticut raw data for consolidation."""

    postal_code = "CT"
    fields = dict(
        company="affected_company",
        location="layoff_location",
        date="warn_date",
        jobs="number_workers",
    )
    date_format = ["%m/%d/%Y", "%m/%d/%y"]
    jobs_corrections = {
        "up to 703": 703,
        "18; 87": 105,
        "724 across U.S. including 49 from Ridgefield CT location": 49,
        "Not Provided": None,
        "Not Indicated": None,
        "Possibly 50+": 50,
        "Not indicated": None,
        "12; 6; 5": 23,
        "182; additional 21 on reduced hours": 182,
        "78; additional 13 on reduced hours": 78,
        "124; additional 30 on reduced hours": 124,
        "Not reported on WARN notice": None,
        "Not provided": None,
    }

    def transform_date(self, value: str) -> typing.Optional[str]:
        """Transform a raw date string into a date object.

        Args:
            value (str): The raw date string provided by the source

        Returns: A date object ready for consolidation. Or, if the date string is invalid, a None.
        """
        value = value.strip().split()[-1].strip()
        return super().transform_date(value)



from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Georgia raw data for consolidation."""

    postal_code = "GA"
    fields = dict(
        company="Company Name",
        location="City",
        date="Separation Date",
        jobs="Est. Impact",
    )
    date_format = "%m/%d/%Y"



from datetime import datetime

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Maryland raw data for consolidation."""

    postal_code = "MD"
    fields = dict(
        company="Company",
        location="Location",
        date="Notice Date",
        jobs="Total Employees",
    )
    date_format = "%m/%d/%Y"
    date_corrections = {
        "4/1/2020 (REVISED) 10/22/2020": datetime(2020, 4, 1),
        "3/3020/17": datetime(2017, 3, 20),
        "5/62011": datetime(2011, 5, 6),
        "2/1/2010 to 2/28/2010": datetime(2010, 2, 1),
        "NOTE: possible event 10/22/2010": datetime(2010, 10, 22),
    }
    jobs_corrections = {
        "103 (REVISED) 10/22/2020 108": 103,
        "1100-1200 (MDDCVA)": 1100,
        "TBD": None,
        "approx. 150": 150,
        "Initially 110 (Possibly as high as 160)": 110,
        "Unknown": None,
        "Starting with 35 leading up to several hundred by December 2013": 35,
        "Total 106 (at this time number impacted at this location is unknown)": 106,
        "Not sure of the number of impacted workers in MD at this time": None,
        "Unknown at this time": None,
        "60-70 in Maryland": 60,
        "unknown at this time": None,
        "8 additional": 8,
        "Total 35 (At this time number impacted at this location is unknown.)": 35,
        "Not known": None,
        "N/A": None,
    }



from datetime import datetime

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Rhode Island raw data for consolidation."""

    postal_code = "RI"
    fields = dict(
        company="Company Name (* Denotes Covid 19 Related WARN)",
        location="Location of Layoffs",
        date="WARN Date",
        jobs="Number Affected",
    )
    date_format = "%Y-%m-%d %H:%M:%S"
    date_corrections = {
        "2108-10-23 00:00:00": datetime(2018, 10, 23),
        "2108-11-01 00:00:00": datetime(2018, 11, 1),
    }
    jobs_corrections = {
        "---": None,
        "54 Union 3 Non Union": 57,
        "190 company with an additional 100 contracted": 290,
        "60-80": 60,
        "additional 16": 16,
    }

    def transform_company(self, value: str) -> str:
        """Transform a raw company name.

        Args:
            value (str): The raw company string provided by the source

        Returns: A string object ready for consolidation.
        """
        return value.strip().replace("*", "")



from datetime import datetime

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Alaska raw data for consolidation."""

    postal_code = "AK"
    fields = dict(
        company="Company",
        location="Location",
        date="Notice Date",
        jobs="Employees Affected",
    )
    date_format = "%m/%d/%y"
    date_corrections = {"9/30/20*": datetime(2020, 9, 30)}
    jobs_corrections = {
        "Up to 300": 300,
    }



import re
import typing

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Kentucky raw data for consolidation."""

    postal_code = "KY"
    fields = dict(
        company="Company Name",
        location=lambda row: row["Location"]
        or row["County: Local  Name"]
        or row["County"],
        date="Date Received",
        jobs="Employees",
    )
    date_format = "%Y-%m-%d %H:%M:%S"
    minimum_year = 1997
    date_corrections = {
        "43490.0": None,
        "N/A": None,
        "November": None,
    }
    jobs_corrections = {
        "?": None,
        "N/A": None,
        "See W.A.R.N.": None,
        "74 fulltime and 184 parttime": 74,
        "Reduction from 13 to 1": 12,
        "See WARN": None,
        "in WARN": None,
        "Unknown": None,
        "TBD": None,
    }

    def transform_jobs(self, value: str) -> typing.Optional[int]:
        """Transform a raw jobs number into an integer.

        Args:
            value (str): A raw jobs number provided by the source

        Returns: An integer number ready for consolidation. Or, if the value is invalid, a None.
        """
        value = value.split("-")[0].strip()
        value = value.replace("+/-", "")
        value = value.replace("+/", "").strip()
        value = value.replace("+", "").strip()
        value = re.split(" {5,}", value)[0].strip()
        return super().transform_jobs(value)



from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Illinois raw data for consolidation."""

    postal_code = "IL"
    fields = dict(
        company="Location Name",
        location=lambda row: f"{row['Location Address']} {row['Location City']}, {row['Location State']} {row['Location Zipcode']}".strip(),
        date=lambda row: row["Initial Date Reported"] or row["Notify Date"],
        jobs="Total # of Employees",
    )
    date_format = "%Y-%m-%d %H:%M:%S"
    minimum_year = 1987
    maximum_jobs = 100000



from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Alabama raw data for consolidation."""

    postal_code = "AL"
    fields = dict(
        company="Company",
        location="City",
        date="Initial Report Date",
        jobs="Planned # Affected Employees",
    )
    date_format = "%m/%d/%Y"



from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Montana raw data for consolidation."""

    postal_code = "MT"
    fields = dict(
        company="Name of Company",
        location="County",
        date="Date of Notice",
        jobs="Number of Employees Affected",
    )
    date_format = ("%Y-%m-%d %H:%M:%S", "%Y-%m-%d")
    jobs_corrections = {
        "Not noted": None,
        "MT # unknown": None,
    }



from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Washington raw data for consolidation."""

    postal_code = "WA"
    fields = dict(
        company="Company",
        location="Location",
        date="Layoff Start Date",
        jobs="# of Workers",
    )
    date_format = "%m/%d/%Y"



import typing
from datetime import datetime

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Indiana raw data for consolidation."""

    postal_code = "IN"
    fields = dict(
        company="Company",
        location="City",
        date="Notice Date",
        jobs="Affected Workers",
    )
    date_format = ["%m/%d/%Y", "%m/%d/%y"]
    jobs_corrections = {
        "97 (in MI)0 (in IN)": 0,
        "100+": 100,
        "62 MAY be affected": 62,
        "5 in Indiana": 5,
        "Unknown": None,
        "75 in Indiana": 75,
        "40-50": 40,
        "100-130": 100,
        "4 Hoosiers": 4,
        "Undisclosed at this time": None,
        "500 Nationwide": None,
        "NA": None,
        "103 (REVISED) 10/22/2020 108": 103,
    }
    date_corrections = {
        "01/30/1202": datetime(2012, 1, 30),
    }

    def prep_row_list(
        self, row_list: typing.List[typing.Dict]
    ) -> typing.List[typing.Dict]:
        """Make necessary transformations to the raw row list prior to transformation.

        Args:
            row_list (list): A list of raw rows of data from the source.

        Returns: The row list minus empty records
        """
        # Do the standard stuff
        row_list = super().prep_row_list(row_list)

        # Cut rows with data-free revisions
        return [r for r in row_list if r["Affected Workers"] != "N/A"]



from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Nebraska raw data for consolidation."""

    postal_code = "NE"
    fields = dict(
        company="Company",
        location="City",
        date="Date",
        jobs="Jobs Affected",
    )
    date_format = "%m/%d/%Y"
    jobs_corrections = {
        "100+": 100,
        "5-9": 5,
        "3-5": 3,
        "a few": 1,
    }



from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform New York raw data for consolidation."""

    postal_code = "NY"
    fields = dict(
        company="Company",
        location="City",
        date="Notice Date",
        jobs="Number Affected",
    )
    date_format = "%Y-%m-%d %H:%M:%S"



import typing
from datetime import datetime

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform Idaho raw data for consolidation."""

    postal_code = "ID"
    fields = dict(
        company="Company",
        location="City",
        date="Date of Letter",
        jobs="No. of Employees Affected",
    )
    date_format = "%m/%d/%Y"
    date_corrections = {
        "2/19/219": datetime(2019, 2, 19),
    }
    jobs_corrections = {
        "8 in ID": 8,
        "17 in ID": 17,
    }

    def transform_date(self, value: str) -> typing.Optional[str]:
        """Transform a raw date string into a date object.

        Args:
            value (str): The raw date string provided by the source

        Returns: A date object ready for consolidation. Or, if the date string is invalid, a None.
        """
        # A little custom clean up based on the weird stuff from this source
        value = value.strip().split()[0].replace(",", "").strip()

        # The same old stuff
        return super().transform_date(value)



from datetime import datetime

from ..schema import BaseTransformer


class Transformer(BaseTransformer):
    """Transform California raw data for consolidation."""

    postal_code = "CA"
    fields = dict(
        company="company",
        location=lambda row: row["city"] or row["address"],
        date="notice_date",
        jobs="num_employees",
    )
    date_format = "%m/%d/%Y"
    minimum_year = 2014
    jobs_corrections = {
        # This Tesla layoff number large but correct
        # https://www.cnbc.com/2020/05/13/coronavirus-latest-updates.html
        11083: 11083,
    }
    date_corrections = {
        "09/04/2008": datetime(2018, 9, 4),
    }
